@@ -2,7 +2,7 @@
 """
 OnionShare | https://onionshare.org/
 
-Copyright (C) 2015 Micah Lee <micah@micahflee.com>
+Copyright (C) 2016 Micah Lee <micah@micahflee.com>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,13 +17,8 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-import os, inspect, hashlib, base64, hmac, platform, zipfile, tempfile
-from itertools import izip
-
-# hack to make unicode filenames work (#141)
-import sys
-reload(sys)
-sys.setdefaultencoding("utf-8")
+import sys, os, inspect, hashlib, base64, platform, zipfile, tempfile, math, time
+from random import SystemRandom
 
 
 def get_platform():
@@ -32,54 +27,48 @@ def get_platform():
     """
     return platform.system()
 
-if get_platform() == 'Darwin':
-    # this is hacky, but it ultimate ends up returning the absolute path to
-    # OnionShare.app/Contents/Resources, based on the location of helpers.py
-    helpers_path = os.path.realpath(os.path.abspath(inspect.getfile(inspect.currentframe())))
-    osx_resources_dir = os.path.dirname(os.path.dirname(helpers_path))
-else:
-    osx_resources_dir = None
+
+def get_resource_path(filename):
+    """
+    Returns the absolute path of a resource, regardless of whether OnionShare is installed
+    systemwide, and whether regardless of platform
+    """
+    p = get_platform()
+    if p == 'Linux' and sys.argv and sys.argv[0].startswith('/usr/bin/onionshare'):
+        # OnionShare is installed systemwide in Linux
+        resources_dir = os.path.join(sys.prefix, 'share/onionshare')
+    elif getattr(sys, 'frozen', False): # Check if app is "frozen" with cx_Freeze
+        # http://cx-freeze.readthedocs.io/en/latest/faq.html#using-data-files
+        resources_dir = os.path.join(os.path.dirname(sys.executable), 'resources')
+    else:  # Look for resources directory relative to python file
+        resources_dir = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))), 'resources')
+
+    return os.path.join(resources_dir, filename)
 
 
-def get_onionshare_dir():
+def get_version():
     """
-    Returns the OnionShare directory.
+    Returns the version of OnionShare that is running.
     """
-    if get_platform() == 'Darwin':
-        onionshare_dir = os.path.dirname(__file__)
-    else:
-        onionshare_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
-    return onionshare_dir
-
-
-def get_html_path(filename):
-    """
-    Returns the path of the html files.
-    """
-    p = platform.system()
-    if p == 'Darwin':
-        prefix = os.path.join(osx_resources_dir, 'html')
-    else:
-        prefix = get_onionshare_dir()
-    return os.path.join(prefix, filename)
+    return open(get_resource_path('version.txt')).read().strip()
 
 
 def constant_time_compare(val1, val2):
     """
-    Compares two values in constant time.
-    """
-    _builtin_constant_time_compare = getattr(hmac, 'compare_digest', None)
-    if _builtin_constant_time_compare is not None:
-        return _builtin_constant_time_compare(val1, val2)
+    Returns True if the two strings are equal, False otherwise.
 
-    len_eq = len(val1) == len(val2)
-    if len_eq:
-        result = 0
-        left = val1
-    else:
-        result = 1
-        left = val2
-    for x, y in izip(bytearray(left), bytearray(val2)):
+    The time taken is independent of the number of characters that match.
+
+    For the sake of simplicity, this function executes in constant time only
+    when the two strings have the same length. It short-circuits when they
+    have different lengths.
+
+    From: http://www.levigross.com/2014/02/07/constant-time-comparison-functions-in...-python-haskell-clojure-and-java/
+    """
+    if len(val1) != len(val2):
+        return False
+    result = 0
+    for x, y in zip(val1, val2):
         result |= x ^ y
     return result == 0
 
@@ -90,12 +79,23 @@ def random_string(num_bytes, output_len=None):
     """
     b = os.urandom(num_bytes)
     h = hashlib.sha256(b).digest()[:16]
-    s = base64.b32encode(h).lower().replace('=', '')
+    s = base64.b32encode(h).lower().replace(b'=', b'').decode('utf-8')
     if not output_len:
         return s
     return s[:output_len]
 
 
+def build_slug():
+    """
+    Returns a random string made from two words from the wordlist, such as "deter-trig".
+    """
+    wordlist = open(get_resource_path('wordlist.txt')).read().split('\n')
+    wordlist.remove('')
+
+    r = SystemRandom()
+    return '-'.join(r.choice(wordlist) for x in range(2))
+
+
 def human_readable_filesize(b):
     """
     Returns filesize in a human readable format.
@@ -112,6 +112,44 @@ def human_readable_filesize(b):
     return '{0:.1f} {1:s}'.format(round(b, 1), units[u])
 
 
+def format_seconds(seconds):
+    """Return a human-readable string of the format 1d2h3m4s"""
+    seconds_in_a_minute = 60
+    seconds_in_an_hour = seconds_in_a_minute * 60
+    seconds_in_a_day = seconds_in_an_hour * 24
+
+    days = math.floor(seconds / seconds_in_a_day)
+
+    hour_seconds = seconds % seconds_in_a_day
+    hours = math.floor(hour_seconds / seconds_in_an_hour)
+
+    minute_seconds = hour_seconds % seconds_in_an_hour
+    minutes = math.floor(minute_seconds / seconds_in_a_minute)
+
+    remaining_seconds = minute_seconds % seconds_in_a_minute
+    seconds = math.ceil(remaining_seconds)
+
+    human_readable = []
+    if days > 0:
+        human_readable.append("{}d".format(int(days)))
+    if hours > 0:
+        human_readable.append("{}h".format(int(hours)))
+    if minutes > 0:
+        human_readable.append("{}m".format(int(minutes)))
+    if seconds > 0:
+        human_readable.append("{}s".format(int(seconds)))
+    return ''.join(human_readable)
+
+
+def estimated_time_remaining(bytes_downloaded, total_bytes, started):
+    now = time.time()
+    time_elapsed = now - started  # in seconds
+    download_rate = bytes_downloaded / time_elapsed
+    remaining_bytes = total_bytes - bytes_downloaded
+    eta = remaining_bytes / download_rate
+    return format_seconds(eta)
+
+
 def is_root():
     """
     Returns if user is root.
