@@ -2,7 +2,7 @@
 """
 OnionShare | https://onionshare.org/
 
-Copyright (C) 2015 Micah Lee <micah@micahflee.com>
+Copyright (C) 2016 Micah Lee <micah@micahflee.com>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,24 +19,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 from __future__ import division
 import os, sys, subprocess, inspect, platform, argparse, threading, time, math, inspect, platform
-from PyQt4 import QtCore, QtGui
+from PyQt5 import QtCore, QtWidgets, QtGui
 
-import common
-
-try:
-    import onionshare
-except ImportError:
-    sys.path.append(os.path.abspath(common.onionshare_gui_dir + "/.."))
-    import onionshare
+import onionshare
 from onionshare import strings, helpers, web
 
-from file_selection import FileSelection
-from server_status import ServerStatus
-from downloads import Downloads
-from options import Options
+from .file_selection import FileSelection
+from .server_status import ServerStatus
+from .downloads import Downloads
+from .options import Options
 
 
-class Application(QtGui.QApplication):
+class Application(QtWidgets.QApplication):
     """
     This is Qt's QApplication class. It has been overridden to support threads
     and the quick keyboard shortcut.
@@ -45,7 +39,7 @@ class Application(QtGui.QApplication):
         platform = helpers.get_platform()
         if platform == 'Linux':
             self.setAttribute(QtCore.Qt.AA_X11InitThreads, True)
-        QtGui.QApplication.__init__(self, sys.argv)
+        QtWidgets.QApplication.__init__(self, sys.argv)
         self.installEventFilter(self)
 
     def eventFilter(self, obj, event):
@@ -56,7 +50,7 @@ class Application(QtGui.QApplication):
         return False
 
 
-class OnionShareGui(QtGui.QWidget):
+class OnionShareGui(QtWidgets.QMainWindow):
     """
     OnionShareGui is the main window for the GUI that contains all of the
     GUI elements.
@@ -64,6 +58,8 @@ class OnionShareGui(QtGui.QWidget):
     start_server_finished = QtCore.pyqtSignal()
     stop_server_finished = QtCore.pyqtSignal()
     starting_server_step2 = QtCore.pyqtSignal()
+    starting_server_step3 = QtCore.pyqtSignal()
+    starting_server_error = QtCore.pyqtSignal(str)
 
     def __init__(self, qtapp, app):
         super(OnionShareGui, self).__init__()
@@ -96,9 +92,11 @@ class OnionShareGui(QtGui.QWidget):
         self.file_selection.file_list.files_updated.connect(self.server_status.update)
         self.server_status.url_copied.connect(self.copy_url)
         self.starting_server_step2.connect(self.start_server_step2)
+        self.starting_server_step3.connect(self.start_server_step3)
+        self.starting_server_error.connect(self.start_server_error)
 
         # filesize warning
-        self.filesize_warning = QtGui.QLabel()
+        self.filesize_warning = QtWidgets.QLabel()
         self.filesize_warning.setStyleSheet('padding: 10px 0; font-weight: bold; color: #333333;')
         self.filesize_warning.hide()
 
@@ -109,69 +107,80 @@ class OnionShareGui(QtGui.QWidget):
         self.options = Options(web, self.app)
 
         # status bar
-        self.status_bar = QtGui.QStatusBar()
+        self.status_bar = QtWidgets.QStatusBar()
         self.status_bar.setSizeGripEnabled(False)
+        version_label = QtWidgets.QLabel('v{0:s}'.format(helpers.get_version()))
+        version_label.setStyleSheet('color: #666666; padding: 0 10px;')
+        self.status_bar.addPermanentWidget(version_label)
+        self.setStatusBar(self.status_bar)
 
         # main layout
-        self.layout = QtGui.QVBoxLayout()
+        self.layout = QtWidgets.QVBoxLayout()
         self.layout.addLayout(self.file_selection)
         self.layout.addLayout(self.server_status)
         self.layout.addWidget(self.filesize_warning)
         self.layout.addLayout(self.downloads)
         self.layout.addLayout(self.options)
-        self.layout.addWidget(self.status_bar)
-        self.setLayout(self.layout)
+        central_widget = QtWidgets.QWidget()
+        central_widget.setLayout(self.layout)
+        self.setCentralWidget(central_widget)
         self.show()
 
         # check for requests frequently
         self.timer = QtCore.QTimer()
-        QtCore.QObject.connect(self.timer, QtCore.SIGNAL("timeout()"), self.check_for_requests)
+        self.timer.timeout.connect(self.check_for_requests)
         self.timer.start(500)
 
-    def start_server_step2(self):
-        """
-        Step 2 in starting the onionshare server. This displays the large filesize
-        warning, if applicable.
-        """
-        # warn about sending large files over Tor
-        if web.zip_filesize >= 157286400:  # 150mb
-            self.filesize_warning.setText(strings._("large_filesize", True))
-            self.filesize_warning.show()
-
     def start_server(self):
         """
-        Start the onionshare server. This uses multiple threads to start the Tor hidden
+        Start the onionshare server. This uses multiple threads to start the Tor onion
         server and the web app.
         """
-        # start the hidden service
-        self.status_bar.showMessage(strings._('gui_starting_server1', True))
-        self.app.choose_port()
-        try:
-            self.status_bar.showMessage(strings._('gui_starting_server1', True))
-            self.app.start_hidden_service(gui=True)
-        except onionshare.hs.NoTor as e:
-            alert(e.args[0], QtGui.QMessageBox.Warning)
-            self.server_status.stop_server()
-            self.status_bar.clearMessage()
-            return
+        # Reset web counters
+        web.download_count = 0
+        web.error404_count = 0
+        web.set_gui_mode()
 
-        # start onionshare service in new thread
+        # pick an available local port for the http service to listen on
+        self.app.choose_port()
+
+        # start onionshare http service in new thread
         t = threading.Thread(target=web.start, args=(self.app.port, self.app.stay_open, self.app.transparent_torification))
         t.daemon = True
         t.start()
+        # wait for modules in thread to load, preventing a thread-related cx_Freeze crash
+        time.sleep(0.2)
 
+        # start the onion service in a new thread
+        def start_onion_service(self):
+            self.status_bar.showMessage(strings._('gui_starting_server1', True))
+            try:
+                self.app.start_onion_service()
+                self.starting_server_step2.emit()
+
+            except onionshare.onion.NoTor as e:
+                self.starting_server_error.emit(e.args[0])
+                return
+
+        t = threading.Thread(target=start_onion_service, kwargs={'self': self})
+        t.daemon = True
+        t.start()
+
+    def start_server_step2(self):
+        """
+        Step 2 in starting the onionshare server. Prepare files for serving.
+        """
         # prepare the files for sending in a new thread
         def finish_starting_server(self):
             # prepare files to share
             web.set_file_info(self.file_selection.file_list.filenames)
             self.app.cleanup_filenames.append(web.zip_filename)
-            self.starting_server_step2.emit()
+            self.starting_server_step3.emit()
 
             # wait for hs
-            if not self.app.local_only:
-                if not self.app.hs.supports_ephemeral:
-                    self.status_bar.showMessage(strings._('gui_starting_server3', True))
-                    self.app.hs.wait_for_hs(self.app.onion_host)
+            if not self.app.local_only and not self.app.onion.supports_ephemeral:
+                self.status_bar.showMessage(strings._('gui_starting_server3', True))
+                self.app.onion.wait_for_hs(self.app.onion_host)
 
             # done
             self.start_server_finished.emit()
@@ -181,6 +190,24 @@ class OnionShareGui(QtGui.QWidget):
         t.daemon = True
         t.start()
 
+    def start_server_step3(self):
+        """
+        Step 3 in starting the onionshare server. This displays the large filesize
+        warning, if applicable.
+        """
+        # warn about sending large files over Tor
+        if web.zip_filesize >= 157286400:  # 150mb
+            self.filesize_warning.setText(strings._("large_filesize", True))
+            self.filesize_warning.show()
+
+    def start_server_error(self, error):
+        """
+        If there's an error when trying to start the onion service
+        """
+        alert(error, QtWidgets.QMessageBox.Warning)
+        self.server_status.stop_server()
+        self.status_bar.clearMessage()
+
     def stop_server(self):
         """
         Stop the onionshare server.
@@ -207,7 +234,7 @@ class OnionShareGui(QtGui.QWidget):
             try:
                 r = web.q.get(False)
                 events.append(r)
-            except web.Queue.Empty:
+            except web.queue.Empty:
                 done = True
 
         for event in events:
@@ -217,8 +244,12 @@ class OnionShareGui(QtGui.QWidget):
             elif event["type"] == web.REQUEST_DOWNLOAD:
                 self.downloads.add_download(event["data"]["id"], web.zip_filesize)
 
+            elif event["type"] == web.REQUEST_RATE_LIMIT:
+                self.stop_server()
+                alert(strings._('error_rate_limit'), QtWidgets.QMessageBox.Critical)
+
             elif event["type"] == web.REQUEST_PROGRESS:
-                self.downloads.update_download(event["data"]["id"], web.zip_filesize, event["data"]["bytes"])
+                self.downloads.update_download(event["data"]["id"], event["data"]["bytes"])
 
                 # is the download complete?
                 if event["data"]["bytes"] == web.zip_filesize:
@@ -230,7 +261,7 @@ class OnionShareGui(QtGui.QWidget):
                 self.downloads.cancel_download(event["data"]["id"])
 
             elif event["path"] != '/favicon.ico':
-                self.status_bar.showMessage('{0:s}: {1:s}'.format(strings._('other_page_loaded', True), event["path"]))
+                self.status_bar.showMessage('[#{0:d}] {1:s}: {2:s}'.format(web.error404_count, strings._('other_page_loaded', True), event["path"]))
 
     def copy_url(self):
         """
@@ -244,12 +275,30 @@ class OnionShareGui(QtGui.QWidget):
         """
         self.status_bar.clearMessage()
 
+    def closeEvent(self, e):
+        if self.server_status.status != self.server_status.STATUS_STOPPED:
+            dialog = QtWidgets.QMessageBox()
+            dialog.setWindowTitle("OnionShare")
+            dialog.setText(strings._('gui_quit_warning', True))
+            quit_button = dialog.addButton(strings._('gui_quit_warning_quit', True), QtWidgets.QMessageBox.YesRole)
+            dont_quit_button = dialog.addButton(strings._('gui_quit_warning_dont_quit', True), QtWidgets.QMessageBox.NoRole)
+            dialog.setDefaultButton(dont_quit_button)
+            reply = dialog.exec_()
 
-def alert(msg, icon=QtGui.QMessageBox.NoIcon):
+            # Quit
+            if reply == 0:
+                self.stop_server()
+                e.accept()
+            # Don't Quit
+            else:
+                e.ignore()
+
+
+def alert(msg, icon=QtWidgets.QMessageBox.NoIcon):
     """
     Pop up a message in a dialog window.
     """
-    dialog = QtGui.QMessageBox()
+    dialog = QtWidgets.QMessageBox()
     dialog.setWindowTitle("OnionShare")
     dialog.setWindowIcon(window_icon)
     dialog.setText(msg)
@@ -261,7 +310,8 @@ def main():
     """
     The main() function implements all of the logic that the GUI version of onionshare uses.
     """
-    strings.load_strings()
+    strings.load_strings(helpers)
+    print(strings._('version_string').format(helpers.get_version()))
 
     # start the Qt app
     global qtapp
@@ -288,7 +338,7 @@ def main():
 
     # create the onionshare icon
     global window_icon
-    window_icon = QtGui.QIcon(common.get_image_path('logo.png'))
+    window_icon = QtGui.QIcon(helpers.get_resource_path('images/logo.png'))
 
     # validation
     if filenames:
@@ -308,7 +358,7 @@ def main():
     # clean up when app quits
     def shutdown():
         app.cleanup()
-    qtapp.connect(qtapp, QtCore.SIGNAL("aboutToQuit()"), shutdown)
+    qtapp.aboutToQuit.connect(shutdown)
 
     # launch the gui
     gui = OnionShareGui(qtapp, app)
